'use strict';

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

var _images = require('./images');

var _images2 = _interopRequireDefault(_images);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var CardScratchOff = function (_PureComponent) {
  _inherits(CardScratchOff, _PureComponent);

  function CardScratchOff(props) {
    _classCallCheck(this, CardScratchOff);

    var _this = _possibleConstructorReturn(this, _PureComponent.call(this, props));

    _this.handleGestureStart = function (event) {
      if (event.target === _this.canvas) {
        event.preventDefault();
      }
      _this.isDrawing = true;
      _this.lastPosition = _this.getGesturePosition(_this.canvas, event);
      _this.gestureStartPosition = _this.lastPosition;
    };

    _this.handleGestureEnd = function (event) {
      if (!_this.gestureMoving || _this.gesturePosition.x === _this.gestureStartPosition.x && _this.gesturePosition.y === _this.gestureStartPosition.y) {
        _this.handleClickToScratch();
      }
      _this.gestureMoving = false;
      if (event.target === _this.canvas) {
        event.preventDefault();
      }
      _this.isDrawing = false;
    };

    _this.handleGestureMove = function (event) {
      _this.gestureMoving = true;
      if (event.target === _this.canvas) {
        event.preventDefault();
      }
      _this.gesturePosition = _this.getGesturePosition(_this.canvas, event);
      _this.draw();
      _this.lastPosition = _this.gesturePosition;
    };

    _this.canvasRef = _react2.default.createRef();
    _this.state = {
      // use to show secret content
      isLoadedCover: false
    };
    _this.clickToScratchCount = 0;
    return _this;
  }

  CardScratchOff.prototype.componentDidMount = function componentDidMount() {
    this.setupCanvas();
    this.setupCover();
    this.setupBrush();
    this.setupGestureEvent();
  };

  CardScratchOff.prototype.componentWillUnmount = function componentWillUnmount() {
    this.unregisterGestureEvent();
  };

  CardScratchOff.prototype.setupCanvas = function setupCanvas() {
    // get canvas ref
    var current = this.canvasRef.current;

    // save canvas ref to use later

    this.canvas = current;

    /**
     * get parent's size to set width and height for canvas (full parent)
     * default width/height: 300/150
     */
    var parentNode = this.canvas.parentNode;

    var parentSize = parentNode.getBoundingClientRect && parentNode.getBoundingClientRect();

    var _ref = parentSize || {},
        _ref$width = _ref.width,
        width = _ref$width === undefined ? 0 : _ref$width,
        _ref$height = _ref.height,
        height = _ref$height === undefined ? 0 : _ref$height;

    this.canvas.width = width;
    this.canvas.height = height;

    // save canvas width and height to use later
    this.canvasWidth = width;
    this.canvasHeight = height;

    // save context to control canvas
    this.ctx = this.canvas.getContext('2d');

    // setup anchor point
    this.setupAnchorPoint(width, height);
  };

  CardScratchOff.prototype.setupAnchorPoint = function setupAnchorPoint(canvasWidth, canvasHeight) {
    var _props = this.props,
        numberOfAnchorPointsPerRow = _props.numberOfAnchorPointsPerRow,
        numberOfAnchorPointsPerColumn = _props.numberOfAnchorPointsPerColumn;


    this.anchorPointList = [];

    for (var row = 1; row <= numberOfAnchorPointsPerRow; row++) {
      for (var col = 1; col <= numberOfAnchorPointsPerColumn; col++) {
        this.anchorPointList.push({
          x: ((col - 1) * 2 + 1) * canvasWidth / (numberOfAnchorPointsPerColumn * 2),
          y: ((row - 1) * 2 + 1) * canvasHeight / (numberOfAnchorPointsPerRow * 2)
        });
      }
    }

    // uncomment this to see anchor point
    // this.anchorPointList.forEach((point) =>
    //   Utils.drawPoint(this.ctx, point)
    // );
  };

  CardScratchOff.prototype.setupCover = function setupCover() {
    var _this2 = this;

    var coverSrc = this.props.coverSrc;
    var ctx = this.ctx,
        canvasWidth = this.canvasWidth,
        canvasHeight = this.canvasHeight;

    var cover = new Image();
    cover.src = coverSrc;
    cover.onload = function () {
      ctx.drawImage(cover, 0, 0, canvasWidth, canvasHeight);
      _this2.setState({ isLoadedCover: true });
    };
  };

  CardScratchOff.prototype.setupBrush = function setupBrush() {
    var brushesSrc = this.props.brushesSrc;

    var brushes = new Image();
    brushes.src = brushesSrc;
    this.brushes = brushes;
  };

  CardScratchOff.prototype.setupGestureEvent = function setupGestureEvent() {
    this.isDrawing = false;
    this.gesturePosition = { x: 0, y: 0 };
    this.lastPosition = this.gesturePosition;
    this.registerGestureEvent();
  };

  CardScratchOff.prototype.registerGestureEvent = function registerGestureEvent() {
    var isMobileDevice = this.props.isMobileDevice;

    var canvas = this.canvas;
    var gestureStartEvent = isMobileDevice ? 'touchstart' : 'mousedown';
    var gestureEndEvent = isMobileDevice ? 'touchend' : 'mouseup';
    var gestureMoveEvent = isMobileDevice ? 'touchmove' : 'mousemove';
    canvas.addEventListener(gestureStartEvent, this.handleGestureStart);
    canvas.addEventListener(gestureEndEvent, this.handleGestureEnd);
    canvas.addEventListener(gestureMoveEvent, this.handleGestureMove);
  };

  CardScratchOff.prototype.unregisterGestureEvent = function unregisterGestureEvent() {
    var isMobileDevice = this.props.isMobileDevice;

    var canvas = this.canvas;
    var gestureStartEvent = isMobileDevice ? 'touchstart' : 'mousedown';
    var gestureEndEvent = isMobileDevice ? 'touchend' : 'mouseup';
    var gestureMoveEvent = isMobileDevice ? 'touchmove' : 'mousemove';
    canvas.removeEventListener(gestureStartEvent, this.handleGestureStart);
    canvas.removeEventListener(gestureEndEvent, this.handleGestureEnd);
    canvas.removeEventListener(gestureMoveEvent, this.handleGestureMove);
  };

  CardScratchOff.prototype.getGesturePosition = function getGesturePosition(canvasDom, gestureEvent) {
    var isMobileDevice = this.props.isMobileDevice;

    var rect = canvasDom.getBoundingClientRect();
    return {
      x: (isMobileDevice ? gestureEvent.touches[0].clientX : gestureEvent.clientX) - rect.left,
      y: (isMobileDevice ? gestureEvent.touches[0].clientY : gestureEvent.clientY) - rect.top
    };
  };

  CardScratchOff.prototype.handleRemoveCanvas = function handleRemoveCanvas(canvasDom) {
    var onScratchFinish = this.props.onScratchFinish;

    if (canvasDom.parentNode) {
      canvasDom.parentNode.removeChild(canvasDom);
      onScratchFinish && onScratchFinish();
    }
  };

  CardScratchOff.prototype.handlePercentage = function handlePercentage(currentPoint, width, height) {
    var anchorPointList = this.anchorPointList;
    var _props2 = this.props,
        numberOfAnchorPointsPerRow = _props2.numberOfAnchorPointsPerRow,
        numberOfAnchorPointsPerColumn = _props2.numberOfAnchorPointsPerColumn,
        finishPercent = _props2.finishPercent;

    this.handleRemoveAnchorPoint(currentPoint, width, height);
    var anchorPointListLength = anchorPointList.length;
    var percentage = 100 - anchorPointListLength / (numberOfAnchorPointsPerRow * numberOfAnchorPointsPerColumn) * 100;
    if (percentage > finishPercent) {
      this.handleRemoveCanvas(this.canvas);
    }
  };

  CardScratchOff.prototype.handleRemoveAnchorPoint = function handleRemoveAnchorPoint(point, width, height) {
    var anchorPointList = this.anchorPointList;

    var remainPointList = anchorPointList.filter(function (anchorPoint) {
      return !_utils2.default.isInsideRemovedArea(point, width, height, anchorPoint);
    });
    this.anchorPointList = remainPointList;
  };

  CardScratchOff.prototype.handleClickToScratch = function handleClickToScratch() {
    var _props3 = this.props,
        allowClickToScratch = _props3.allowClickToScratch,
        maxTimesClickToScratch = _props3.maxTimesClickToScratch,
        enableScratch = _props3.enableScratch;

    if (!allowClickToScratch || !enableScratch) {
      return;
    }
    this.clickToScratchCount = this.clickToScratchCount + 1;
    if (this.clickToScratchCount === maxTimesClickToScratch) {
      this.handleRemoveCanvas(this.canvas);
    } else {
      this.drawWithBrushes(this.ctx, {
        x: (this.clickToScratchCount - 1) * this.canvasWidth / maxTimesClickToScratch,
        y: 0
      }, {
        x: this.clickToScratchCount * this.canvasWidth / maxTimesClickToScratch,
        y: this.canvasHeight * 5 / 7
      }, this.brushes, 'destination-out', this.canvasWidth / maxTimesClickToScratch, this.canvasHeight * 5 / 7);
    }
  };

  CardScratchOff.prototype.draw = function draw() {
    var enableScratch = this.props.enableScratch;

    if (!enableScratch) {
      return;
    }
    /**
     * if is "scratching" then "draw"
     * if not "scratching" then do nothing
     */
    if (this.isDrawing) {
      var lastPosition = this.lastPosition,
          gesturePosition = this.gesturePosition;

      this.drawWithBrushes(this.ctx, lastPosition, gesturePosition, this.brushes);
      // this like MS Paint ^^
      // Utils.drawLine(this.ctx, lastPosition, gesturePosition);
    }
  };

  CardScratchOff.prototype.drawWithBrushes = function drawWithBrushes(ctx, startPoint, endPoint, brushes) {
    var drawMode = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'destination-out';

    var dist = _utils2.default.distanceBetween(startPoint, endPoint);
    var angle = _utils2.default.angleBetween(startPoint, endPoint);
    var offsetX = brushes.width / 2;
    var offsetY = brushes.height / 2;

    for (var _len = arguments.length, drawImageArgs = Array(_len > 5 ? _len - 5 : 0), _key = 5; _key < _len; _key++) {
      drawImageArgs[_key - 5] = arguments[_key];
    }

    for (var x, y, i = 0; i < dist; i++) {
      x = startPoint.x + Math.sin(angle) * i - offsetX;
      y = startPoint.y + Math.cos(angle) * i - offsetY;
      ctx.globalCompositeOperation = drawMode;
      ctx.drawImage.apply(ctx, [brushes, x, y].concat(drawImageArgs));
      this.handlePercentage({ x: x, y: y }, brushes.width, brushes.height);
    }
  };

  CardScratchOff.prototype.render = function render() {
    var _props4 = this.props,
        children = _props4.children,
        containerStyleClassname = _props4.containerStyleClassname,
        canvasStyleClassName = _props4.canvasStyleClassName;
    var isLoadedCover = this.state.isLoadedCover;

    return _react2.default.createElement(
      'div',
      { className: containerStyleClassname },
      _react2.default.createElement('canvas', { className: canvasStyleClassName, ref: this.canvasRef }),
      !!isLoadedCover && children
    );
  };

  return CardScratchOff;
}(_react.PureComponent);

CardScratchOff.defaultProps = {
  enableScratch: true,
  allowClickToScratch: false,
  numberOfAnchorPointsPerRow: 3,
  numberOfAnchorPointsPerColumn: 5,
  finishPercent: 70,
  maxTimesClickToScratch: 3,
  onScratchFinish: function onScratchFinish() {
    return console.log('scratch finished!!!!!!');
  },
  coverSrc: _images2.default.scratchCover,
  brushesSrc: _images2.default.scratchBrushes,
  isMobileDevice: 'ontouchstart' in window
};

CardScratchOff.propTypes = process.env.NODE_ENV !== "production" ? {
  enableScratch: _propTypes2.default.bool,
  allowClickToScratch: _propTypes2.default.bool,
  numberOfAnchorPointsPerRow: _propTypes2.default.number,
  numberOfAnchorPointsPerColumn: _propTypes2.default.number,
  finishPercent: _propTypes2.default.number,
  maxTimesClickToScratch: _propTypes2.default.number,
  onScratchFinish: _propTypes2.default.func,
  coverSrc: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]),
  brushesSrc: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]),
  isMobileDevice: _propTypes2.default.bool.isRequired,
  containerStyleClassname: _propTypes2.default.string,
  canvasStyleClassName: _propTypes2.default.string
} : {};

exports.default = CardScratchOff;
module.exports = exports['default'];