'use strict';

exports.__esModule = true;
var distanceBetween = function distanceBetween(point1, point2) {
  return Math.sqrt(Math.pow(point2.x - point1.x, 2) + Math.pow(point2.y - point1.y, 2));
};

var angleBetween = function angleBetween(point1, point2) {
  return Math.atan2(point2.x - point1.x, point2.y - point1.y);
};

var isInsideRemovedArea = function isInsideRemovedArea(point, width, height, anchorPoint) {
  if (anchorPoint.x > point.x && anchorPoint.x < point.x + width && anchorPoint.y > point.y && anchorPoint.y < point.y + height) {
    return true;
  }
  return false;
};

var drawRectangle = function drawRectangle(ctx, point, width, height) {
  var drawMode = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'destination-out';

  ctx.globalCompositeOperation = drawMode;
  ctx.fillRect(point.x, point.y, width, height);
};

var drawPoint = function drawPoint(ctx, point) {
  var color = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'red';
  var drawMode = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'destination-over';

  ctx.beginPath();
  ctx.arc(point.x, point.y, 1, 0, 2 * Math.PI);
  ctx.fillStyle = color;
  ctx.fill();
  ctx.strokeStyle = color;
  ctx.globalCompositeOperation = drawMode;
  ctx.stroke();
};

var drawLine = function drawLine(ctx, startPoint, endPoint) {
  var color = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'red';
  var drawMode = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'source-over';

  ctx.moveTo(startPoint.x, startPoint.y);
  ctx.lineTo(endPoint.x, endPoint.y);
  ctx.strokeStyle = color;
  ctx.globalCompositeOperation = drawMode;
  ctx.stroke();
};

exports.default = {
  distanceBetween: distanceBetween,
  angleBetween: angleBetween,
  isInsideRemovedArea: isInsideRemovedArea,
  drawRectangle: drawRectangle,
  drawPoint: drawPoint,
  drawLine: drawLine
};
module.exports = exports['default'];